const UserModel = require("../models/User");
const MessageRestoreRequest = require("../models/MessageRestoreRequest");
const PasswordReset = require("../models/PasswordReset");
const Joi = require('joi');
const bcrypt = require('bcrypt');
const jwt = require("jsonwebtoken");
const { sendEmail, getEmailTemplate } = require('../utils/EmailTemplate');
const moment = require('moment');

const otpStorage = {};

// Function to generate a random OTP
const generateOTP = () => {
    return Math.floor(1000 + Math.random() * 9000);
};

// get otp 
const getOtp = async (req, res) => {
    const { error, value } = schema.validate(req.body.registerInfo);

    if (error) {
        return res.status(400).json({ error: error.details[0].message });
    }

    const { firstName, lastName, userName, email, mobileNumber, password } = req.body.registerInfo;

    console.log(req.body);
    if (!userName || !email || !mobileNumber || !password) return res.status(400).json("All fields are required...");

    let existUserName = await UserModel.findOne({ where: { userName: userName } });
    let existUserEmail = await UserModel.findOne({ where: { email: email } });

    if (existUserName) {
        return res.status(400).json("Username already exists...");
    }

    if (existUserEmail) {
        return res.status(400).json("Email already exists...");
    }


    try {
        // generate otp
        const otp = generateOTP();
        // Store OTP in memory
        otpStorage[email] = otp;
        const subject = 'Your OTP Code';
        const replacements = { name: userName, otp: otp };
        const htmlContent = getEmailTemplate('email-verification', replacements);
        const response = await sendEmail(email, subject, htmlContent);
        return res.status(200).json({ message: "otp sending successfully", response });

    } catch (error) {
        // Return error response
        console.error('Error sending OTP:', error);
        return res.status(500).json({ error: "OTP sending error", error: error.message });
    }
}

// create token 
const createToken = (id, username, email, role, name) => {
    const jwtkey = process.env.JWT_SECRET_KEY;
    return jwt.sign({ id, username, email, role, name }, jwtkey, { expiresIn: "3d" })
}

// validations
const schema = Joi.object({
    firstName: Joi.string().min(3).max(30).messages({ 'any.required': `First Name field is required.` }).required(),
    lastName: Joi.string().min(3).max(30).messages({ 'any.required': `Last Name field is required.` }).required(),
    userName: Joi.string().min(3).max(30).messages({ 'any.required': `Username field is required.` }).required(),
    email: Joi.string().email().required(),
    mobileNumber: Joi.string().length(10)
        .pattern(/[6-9]{1}[0-9]{9}/)
        .required()
        .messages({
            "any.required": "Mobile number is required.",
            "string.empty": "Mobile number cannot be empty.",
            "string.length": "Mobile number must be exactly 10 digits long.",
            "string.pattern.base": "Mobile number is invalid.",
        }),
    password: Joi.string().min(5).max(20).required(),

});

const loginschema = Joi.object({
    email: Joi.string().email().required(),
    password: Joi.string().required(),
});

// register api
const register = async (req, res) => {
    try {

        const { error, value } = schema.validate(req.body);

        if (error) {
            return res.status(400).json({ error: error.details[0].message });
        }

        const { firstName, lastName, userName, email, mobileNumber, password } = req.body;


        if (!userName || !email || !mobileNumber || !password) return res.status(400).json("All fields are required...");

        let existUserName = await UserModel.findOne({ where: { userName: req.body.userName } });
        let existUserEmail = await UserModel.findOne({ where: { email: req.body.email } });

        if (existUserName) {
            return res.status(400).json("Username already exists...");
        }

        if (existUserEmail) {
            return res.status(400).json("Email already exists...");
        }



        const salt = await bcrypt.genSalt(10);
        const bCpassword = await bcrypt.hash(password, salt);
        const user = await UserModel.create({
            firstName,
            lastName,
            username: userName,
            email,
            mobileNumber,
            password: bCpassword,
            role: 3,
        });

        const token = createToken(user.id, user.username, user.email, user.role, user.firstName + ' ' + user.lastName);
        const userId = user.id;
        const id = user.id;
        const role = user.role;
        const userData = {
            id,
            userId,
            firstName,
            lastName,
            username: userName,
            email,
            mobileNumber,
            token,
            role,
        }
        res.status(200).json(userData)
    } catch (error) {
        res.status(500).json({ error: error })
    }

}

// Backend code for verifying OTP and registering user
const verifyOtpAndRegister = async (req, res) => {
    try {
        const { email, otp } = req.body;

        // Retrieve the stored OTP for the email
        const storedOtp = String(otpStorage[email]);

        // Verify if the received OTP matches the stored OTP
        if (storedOtp && storedOtp === otp) {
            // Clear the stored OTP after successful verification
            delete otpStorage[email];

            // Return success response
            return res.status(200).json({ success: true, message: "OTP verification successful" });
        } else {
            // Return error response for OTP mismatch
            return res.status(400).json({ success: false, error: "Invalid OTP" });
        }
    } catch (error) {
        // Handle error
        console.error('Error verifying OTP:', error);
        return res.status(500).json({ success: false, error: "Internal server error" });
    }
}


const login = async (req, res) => {
    const { email, password } = req.body;
    try {

        const { error, value } = loginschema.validate(req.body);

        if (error) {
            console.log('login error', error.details[0])
            return res.status(400).json({ error: error.details[0].message });
        }

        const user = await UserModel.findOne({ where: { email } });

        if (!user) {
            return res.status(400).json("Invalid Email or Password");
        }

        const isValidPassword = await bcrypt.compare(password, user.password);

        if (!isValidPassword) {
            return res.status(400).json("Invalid Email or Password");
        }
        const payload = {
            userId: user.id,
        };
        const refreshToken = jwt.sign(payload, process.env.JWT_SECRET_KEY, { expiresIn: '7d' });

        await UserModel.update({ refreshToken: refreshToken }, { where: { id: user.id } });

        const token = createToken(user.id, user.username, user.email, user.role, user.firstName + ' ' + user.lastName);

        const userData = {
            id: user.id,
            username: user.username, // Assuming you have a 'name' field in your user model
            firstName: user.firstName,
            lastName: user.lastName,
            email: user.email,
            mobileNumber: user.mobileNumber,
            type: user.userType,
            image: user.image,
            refreshToken: user.refreshToken,
            apiToken: user.apiToken,
            token,
            role: user.role,
        };

        res.status(200).json(userData);

    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const generateToken = async (req, res) => {
    const { refreshToken } = req.body; // Assuming the parameter is in the URL
    console.log(refreshToken);
    try {
        const user = await UserModel.findOne({ where: { refreshToken } });
        if (!user) {
            return res.status(404).json({ error: "User not found" });
        }

        // Customize the user data as needed
        const token = createToken(user.id, user.username, user.email, user.role, user.firstName + ' ' + user.lastName);

        const userData = {
            id: user.id,
            firstName: user.firstName,
            lastName: user.lastName,
            username: user.username, // Assuming you have a 'name' field in your user model
            email: user.email,
            mobileNumber: user.mobileNumber,
            type: user.userType,
            image: user.image,
            refreshToken: user.refreshToken,
            token,
            role: user.role,
        };

        res.status(200).json(userData);

    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const findUser = async (req, res) => {
    const id = req.params.id; // Assuming the parameter is in the URL
    try {
        const user = await UserModel.findOne({ where: { id } });
        if (!user) {
            return res.status(404).json({ error: "User not found" });
        }

        // Customize the user data as needed
        const userData = {
            id: user.id,
            username: user.username,
            firstName: user.firstName,
            lastName: user.lastName,
            email: user.email,
            mobileNumber: user.mobileNumber,
            image: user.image,
            role: user.role,
        };

        res.status(200).json(userData);

    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const searchUser = async (req, res) => {

    try {
        const { username } = req.body;

        // Check if userName is provided
        if (!username) {
            return res.status(400).json({ error: 'userName parameter is required' });
        }

        // Find users with matching userName
        const users = await UserModel.findOne({ where: { username } });

        // Customize the user data as needed
        // const userArray = users.map(user => ({
        //     id: user.id,
        //     userName: user.userName,
        //     email: user.email,
        //     mobileNumber: user.mobileNumber,
        //     role: user.role,
        //     // type: user.userType,
        // }));

        res.status(200).json(users);

    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const getAllUsers = async (req, res) => {
    try {
        const users = await UserModel.findAll();

        // Customize the user data as needed
        const userArray = users.map(user => ({
            id: user.id,
            userName: user.userName,
            email: user.email,
            mobileNumber: user.mobileNumber,
            type: user.userType,
            image: user.image,
            role: user.role,
        }));

        res.status(200).json(userArray);

    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

// Function to handle profile image upload
const uploadProfileImage = async (req, res) => {
    try {

        // Get the user ID from the request
        const userId = req.params.userId;
        const { imageUrl } = req.body

        // Update the user's profile image path in the database
        await UserModel.update({ image: imageUrl }, { where: { id: userId } });

        // Send a success response
        res.status(200).json({ message: 'Profile image uploaded successfully' });
    } catch (error) {
        console.error('Error uploading profile image:', error);
        res.status(500).json({ error: 'Internal server error', error });
    }
};


const editProfile = async (req, res) => {
    try {
        const userId = req.params.userId;

        const { firstName, lastName, email, mobileNumber, oldPassword, newPassword } = req.body;

        const fieldsToUpdate = {
            firstName,
            lastName,
            email,
            mobileNumber,
        };
        if (newPassword) {
            const userPss = await UserModel.findOne({ where: { id: userId } });
            if (!userPss) {
                return res.status(400).json("Invalid Password");
            }
            const isValidPassword = await bcrypt.compare(oldPassword, userPss.password);
            if (!isValidPassword) {
                return res.status(400).json({ error: "Old Password is invalid" });
            }
            const salt = await bcrypt.genSalt(10);
            fieldsToUpdate.password = await bcrypt.hash(newPassword, salt);
        }
        if (req.files) {

            const file = req.files.image;
            const fileSize = req.files.image.size;

            if (fileSize < (5 * 1024 * 1024)) {
                let uploadPath = 'upload/images/' + file.name;
                file.mv(uploadPath, async function (err) {
                    if (err) {
                        return res.status(500).json({
                            success: 0,
                            message: 'Error uploading file',
                            error: err.message // Include error message in the response
                        });
                    }

                    fieldsToUpdate.image = `/images/${file.name}`;
                    await UserModel.update(
                        fieldsToUpdate,
                        { where: { id: userId } }
                    );

                    const user = await UserModel.findOne({ where: { id: userId } });
                    const token = createToken(user.id, user.username, user.email, user.role, user.firstName + ' ' + user.lastName);

                    const userData = {
                        id: user.id,
                        firstName: user.firstName,
                        lastName: user.lastName,
                        username: user.username,
                        email: user.email,
                        mobileNumber: user.mobileNumber,
                        type: user.userType,
                        image: user.image,
                        refreshToken: user.refreshToken,
                        token,
                        role: user.role,
                    };
                    // Send a success response
                    res.status(200).json({ success: true, message: 'Profile updated successfully', user: userData });
                });
            }
        }
        else {
            await UserModel.update(
                fieldsToUpdate,
                { where: { id: userId } }
            );

            const user = await UserModel.findOne({ where: { id: userId } });
            const token = createToken(user.id, user.username, user.email, user.role, user.firstName + ' ' + user.lastName);

            const userData = {
                id: user.id,
                firstName: user.firstName,
                lastName: user.lastName,
                username: user.username,
                email: user.email,
                mobileNumber: user.mobileNumber,
                type: user.userType,
                image: user.image,
                refreshToken: user.refreshToken,
                token,
                role: user.role,
            };
            // Send a success response
            res.status(200).json({ success: true, message: 'Profile updated successfully', user: userData });
        }
    } catch (error) {
        console.error('Error editing profile:', error);
        res.status(500).json({ error: error });
    }
}

const deleteUserProfile = async (req, res) => {

    try {

        // Get the user ID from the request
        const userId = req.params.userId;

        // Find the user by ID and delete it
        const deletedUser = await UserModel.destroy({ where: { id: userId } });

        // Check if the user was found and deleted
        if (deletedUser === 0) {
            return res.status(404).json({ error: 'User not found' });
        }

        // Send a success response
        res.status(200).json({ message: 'User profile deleted successfully' });

    } catch (error) {
        return res.status(500).json({ error: 'Internak server error', error });
    }
}

const forgotPassword = async (req, res) => {
    const { email, newPassword } = req.body;

    try {
        // Check if the user with the given email exists
        const user = await UserModel.findOne({ where: { email: email } });

        // If user does not exist, return error
        if (!user) {
            return res.status(404).json({ error: 'User with this email does not exist' });
        }

        try {
            // generate otp
            const subject = 'Reset Password';
            const jwtkey = process.env.JWT_SECRET_KEY;
            const APP_URL = process.env.APP_URL;

            const token = generateRandomString(50);
            await PasswordReset.create({
                email,
                token
            });
            const replacements = { name: user.username, link: `${APP_URL}/chat/reset-password/${token}` };
            const htmlContent = getEmailTemplate('email-reset-password', replacements);

            const response = await sendEmail(email, subject, htmlContent);
            return res.status(200).json({ message: "Reset Link sent to your email", response });

        } catch (error) {
            // Return error response
            console.error('Error sending Reset Link:', error);
            return res.status(500).json({ error: "Reset Link sending error", error: error.message });
        }

    } catch (error) {
        console.error('Error updating password:', error);
        res.status(500).json({ error: 'Internal server error' });
    }
};
const resetPasswordTokenValidate = async (req, res) => {
    const { token } = req.body;

    try {
        // Check if the user with the given email exists
        const password = await PasswordReset.findOne({ where: { token: token } });

        // If user does not exist, return error
        if (!password) {
            res.status(302).json({ error: 'This link is expired' });
        }
        if (differenceInMinutes(password.createdAt, moment()) > 60) {
            res.status(302).json({ error: 'This link is expired' });
        }
        res.status(200).json({ success: true, message: 'validate', time: differenceInMinutes() });
    } catch (error) {
        console.error('Error updating password:', error);
        res.status(500).json({ error: 'Internal server error' });
    }
};
const resetPassword = async (req, res) => {
    const { token, newPassword } = req.body;

    try {
        // Check if the user with the given email exists
        const password = await PasswordReset.findOne({ where: { token: token } });
        if (!password) {
            res.status(404).json({ error: 'This link is expired' });
        }
        const email = password.email;
        const user = await UserModel.findOne({ where: { email: email } });

        // If user does not exist, return error
        if (!user) {
            return res.status(404).json({ error: 'User with this email does not exist' });
        }

        const salt = await bcrypt.genSalt(10);
        const hashedPassword = await bcrypt.hash(newPassword, salt);

        // Update the user's password in the database
        await UserModel.update({ password: hashedPassword }, { where: { email: email } });

        // Send a success response
        res.status(200).json({ success: true, message: 'Password updated successfully' });
    } catch (error) {
        console.error('Error updating password:', error);
        res.status(500).json({ error: 'Internal server error' + error.message });
    }
};
const messageRestoreRequest = async (req, res) => {

    try {

        const { startDate, endDate, chatUserId } = req.body;

        // Get the user ID from the request
        const userId = req.userId;

        // Find the user by ID and delete it
        const request = await MessageRestoreRequest.create({
            user_id: userId,
            chat_user_id: chatUserId,
            start_date: startDate,
            end_date: endDate
        });

        // Send a success response
        res.status(200).json({ message: 'Messages Request submited successfully, it will approve with in 24 hours' });

    } catch (error) {
        return res.status(500).json({ error: 'Internak server error', error });
    }
}
function differenceInMinutes(timestamp1, timestamp2) {
    const moment1 = moment(timestamp1);
    const moment2 = moment(timestamp2);
    const diffInMinutes = Math.abs(moment1.diff(moment2, 'minutes')); // Difference in minutes
    return diffInMinutes;
}
const generateRandomString = (length) => {
    const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let result = '';
    for (let i = 0; i < length; i++) {
        result += characters.charAt(Math.floor(Math.random() * characters.length));
    }
    return result;
};
module.exports = {
    register,
    login,
    findUser,
    getAllUsers,
    searchUser,
    uploadProfileImage,
    editProfile,
    deleteUserProfile,
    getOtp,
    verifyOtpAndRegister,
    forgotPassword,
    generateToken,
    messageRestoreRequest,
    resetPasswordTokenValidate,
    resetPassword
}

